// Copyright 2025 Matt Borland
// Distributed under the Boost Software License, Version 1.0.
// https://www.boost.org/LICENSE_1_0.txt

#ifndef BOOST_DECIMAL_DETAIL_INT128_INT128_HPP
#define BOOST_DECIMAL_DETAIL_INT128_INT128_HPP

#include <boost/decimal/detail/int128/detail/fwd.hpp>
#include <boost/decimal/detail/int128/detail/int128_imp.hpp>
#include <boost/decimal/detail/int128/detail/uint128_imp.hpp>
#include <boost/decimal/detail/int128/detail/conversions.hpp>

#endif // BOOST_DECIMAL_DETAIL_INT128_INT128_HPP
