//
// Copyright (c) 2023 alandefreitas (alandefreitas@gmail.com)
//
// Distributed under the Boost Software License, Version 1.0.
// https://www.boost.org/LICENSE_1_0.txt
//

#ifndef BOOST_URL_SRC_HPP
#define BOOST_URL_SRC_HPP

/*

This file used to be used as a form of library integration
to allow the library to be compiled as a single target.

This form of integration is discontinued.

Please refer to the following link for instructions:
https://www.boost.org/doc/libs/master/libs/url/doc/html/url/overview.html

*/
#error "src.hpp is discontinued and should not be used. Please use the compiled library instead."

#endif