# /* **************************************************************************
#  *                                                                          *
#  *     (C) Copyright Paul Mensonides 2002.
#  *     Distributed under the Boost Software License, Version 1.0. (See
#  *     accompanying file LICENSE_1_0.txt or copy at
#  *     http://www.boost.org/LICENSE_1_0.txt)
#  *                                                                          *
#  ************************************************************************** */
#
# /* Revised by Edward Diener (2020) */
#
# /* See http://www.boost.org for most recent version. */
#
# ifndef BOOST_PREPROCESSOR_SEQ_DETAIL_SPLIT_512_HPP
# define BOOST_PREPROCESSOR_SEQ_DETAIL_SPLIT_512_HPP
#
# define BOOST_PP_SEQ_SPLIT_257(x) (x) BOOST_PP_SEQ_SPLIT_256
# define BOOST_PP_SEQ_SPLIT_258(x) (x) BOOST_PP_SEQ_SPLIT_257
# define BOOST_PP_SEQ_SPLIT_259(x) (x) BOOST_PP_SEQ_SPLIT_258
# define BOOST_PP_SEQ_SPLIT_260(x) (x) BOOST_PP_SEQ_SPLIT_259
# define BOOST_PP_SEQ_SPLIT_261(x) (x) BOOST_PP_SEQ_SPLIT_260
# define BOOST_PP_SEQ_SPLIT_262(x) (x) BOOST_PP_SEQ_SPLIT_261
# define BOOST_PP_SEQ_SPLIT_263(x) (x) BOOST_PP_SEQ_SPLIT_262
# define BOOST_PP_SEQ_SPLIT_264(x) (x) BOOST_PP_SEQ_SPLIT_263
# define BOOST_PP_SEQ_SPLIT_265(x) (x) BOOST_PP_SEQ_SPLIT_264
# define BOOST_PP_SEQ_SPLIT_266(x) (x) BOOST_PP_SEQ_SPLIT_265
# define BOOST_PP_SEQ_SPLIT_267(x) (x) BOOST_PP_SEQ_SPLIT_266
# define BOOST_PP_SEQ_SPLIT_268(x) (x) BOOST_PP_SEQ_SPLIT_267
# define BOOST_PP_SEQ_SPLIT_269(x) (x) BOOST_PP_SEQ_SPLIT_268
# define BOOST_PP_SEQ_SPLIT_270(x) (x) BOOST_PP_SEQ_SPLIT_269
# define BOOST_PP_SEQ_SPLIT_271(x) (x) BOOST_PP_SEQ_SPLIT_270
# define BOOST_PP_SEQ_SPLIT_272(x) (x) BOOST_PP_SEQ_SPLIT_271
# define BOOST_PP_SEQ_SPLIT_273(x) (x) BOOST_PP_SEQ_SPLIT_272
# define BOOST_PP_SEQ_SPLIT_274(x) (x) BOOST_PP_SEQ_SPLIT_273
# define BOOST_PP_SEQ_SPLIT_275(x) (x) BOOST_PP_SEQ_SPLIT_274
# define BOOST_PP_SEQ_SPLIT_276(x) (x) BOOST_PP_SEQ_SPLIT_275
# define BOOST_PP_SEQ_SPLIT_277(x) (x) BOOST_PP_SEQ_SPLIT_276
# define BOOST_PP_SEQ_SPLIT_278(x) (x) BOOST_PP_SEQ_SPLIT_277
# define BOOST_PP_SEQ_SPLIT_279(x) (x) BOOST_PP_SEQ_SPLIT_278
# define BOOST_PP_SEQ_SPLIT_280(x) (x) BOOST_PP_SEQ_SPLIT_279
# define BOOST_PP_SEQ_SPLIT_281(x) (x) BOOST_PP_SEQ_SPLIT_280
# define BOOST_PP_SEQ_SPLIT_282(x) (x) BOOST_PP_SEQ_SPLIT_281
# define BOOST_PP_SEQ_SPLIT_283(x) (x) BOOST_PP_SEQ_SPLIT_282
# define BOOST_PP_SEQ_SPLIT_284(x) (x) BOOST_PP_SEQ_SPLIT_283
# define BOOST_PP_SEQ_SPLIT_285(x) (x) BOOST_PP_SEQ_SPLIT_284
# define BOOST_PP_SEQ_SPLIT_286(x) (x) BOOST_PP_SEQ_SPLIT_285
# define BOOST_PP_SEQ_SPLIT_287(x) (x) BOOST_PP_SEQ_SPLIT_286
# define BOOST_PP_SEQ_SPLIT_288(x) (x) BOOST_PP_SEQ_SPLIT_287
# define BOOST_PP_SEQ_SPLIT_289(x) (x) BOOST_PP_SEQ_SPLIT_288
# define BOOST_PP_SEQ_SPLIT_290(x) (x) BOOST_PP_SEQ_SPLIT_289
# define BOOST_PP_SEQ_SPLIT_291(x) (x) BOOST_PP_SEQ_SPLIT_290
# define BOOST_PP_SEQ_SPLIT_292(x) (x) BOOST_PP_SEQ_SPLIT_291
# define BOOST_PP_SEQ_SPLIT_293(x) (x) BOOST_PP_SEQ_SPLIT_292
# define BOOST_PP_SEQ_SPLIT_294(x) (x) BOOST_PP_SEQ_SPLIT_293
# define BOOST_PP_SEQ_SPLIT_295(x) (x) BOOST_PP_SEQ_SPLIT_294
# define BOOST_PP_SEQ_SPLIT_296(x) (x) BOOST_PP_SEQ_SPLIT_295
# define BOOST_PP_SEQ_SPLIT_297(x) (x) BOOST_PP_SEQ_SPLIT_296
# define BOOST_PP_SEQ_SPLIT_298(x) (x) BOOST_PP_SEQ_SPLIT_297
# define BOOST_PP_SEQ_SPLIT_299(x) (x) BOOST_PP_SEQ_SPLIT_298
# define BOOST_PP_SEQ_SPLIT_300(x) (x) BOOST_PP_SEQ_SPLIT_299
# define BOOST_PP_SEQ_SPLIT_301(x) (x) BOOST_PP_SEQ_SPLIT_300
# define BOOST_PP_SEQ_SPLIT_302(x) (x) BOOST_PP_SEQ_SPLIT_301
# define BOOST_PP_SEQ_SPLIT_303(x) (x) BOOST_PP_SEQ_SPLIT_302
# define BOOST_PP_SEQ_SPLIT_304(x) (x) BOOST_PP_SEQ_SPLIT_303
# define BOOST_PP_SEQ_SPLIT_305(x) (x) BOOST_PP_SEQ_SPLIT_304
# define BOOST_PP_SEQ_SPLIT_306(x) (x) BOOST_PP_SEQ_SPLIT_305
# define BOOST_PP_SEQ_SPLIT_307(x) (x) BOOST_PP_SEQ_SPLIT_306
# define BOOST_PP_SEQ_SPLIT_308(x) (x) BOOST_PP_SEQ_SPLIT_307
# define BOOST_PP_SEQ_SPLIT_309(x) (x) BOOST_PP_SEQ_SPLIT_308
# define BOOST_PP_SEQ_SPLIT_310(x) (x) BOOST_PP_SEQ_SPLIT_309
# define BOOST_PP_SEQ_SPLIT_311(x) (x) BOOST_PP_SEQ_SPLIT_310
# define BOOST_PP_SEQ_SPLIT_312(x) (x) BOOST_PP_SEQ_SPLIT_311
# define BOOST_PP_SEQ_SPLIT_313(x) (x) BOOST_PP_SEQ_SPLIT_312
# define BOOST_PP_SEQ_SPLIT_314(x) (x) BOOST_PP_SEQ_SPLIT_313
# define BOOST_PP_SEQ_SPLIT_315(x) (x) BOOST_PP_SEQ_SPLIT_314
# define BOOST_PP_SEQ_SPLIT_316(x) (x) BOOST_PP_SEQ_SPLIT_315
# define BOOST_PP_SEQ_SPLIT_317(x) (x) BOOST_PP_SEQ_SPLIT_316
# define BOOST_PP_SEQ_SPLIT_318(x) (x) BOOST_PP_SEQ_SPLIT_317
# define BOOST_PP_SEQ_SPLIT_319(x) (x) BOOST_PP_SEQ_SPLIT_318
# define BOOST_PP_SEQ_SPLIT_320(x) (x) BOOST_PP_SEQ_SPLIT_319
# define BOOST_PP_SEQ_SPLIT_321(x) (x) BOOST_PP_SEQ_SPLIT_320
# define BOOST_PP_SEQ_SPLIT_322(x) (x) BOOST_PP_SEQ_SPLIT_321
# define BOOST_PP_SEQ_SPLIT_323(x) (x) BOOST_PP_SEQ_SPLIT_322
# define BOOST_PP_SEQ_SPLIT_324(x) (x) BOOST_PP_SEQ_SPLIT_323
# define BOOST_PP_SEQ_SPLIT_325(x) (x) BOOST_PP_SEQ_SPLIT_324
# define BOOST_PP_SEQ_SPLIT_326(x) (x) BOOST_PP_SEQ_SPLIT_325
# define BOOST_PP_SEQ_SPLIT_327(x) (x) BOOST_PP_SEQ_SPLIT_326
# define BOOST_PP_SEQ_SPLIT_328(x) (x) BOOST_PP_SEQ_SPLIT_327
# define BOOST_PP_SEQ_SPLIT_329(x) (x) BOOST_PP_SEQ_SPLIT_328
# define BOOST_PP_SEQ_SPLIT_330(x) (x) BOOST_PP_SEQ_SPLIT_329
# define BOOST_PP_SEQ_SPLIT_331(x) (x) BOOST_PP_SEQ_SPLIT_330
# define BOOST_PP_SEQ_SPLIT_332(x) (x) BOOST_PP_SEQ_SPLIT_331
# define BOOST_PP_SEQ_SPLIT_333(x) (x) BOOST_PP_SEQ_SPLIT_332
# define BOOST_PP_SEQ_SPLIT_334(x) (x) BOOST_PP_SEQ_SPLIT_333
# define BOOST_PP_SEQ_SPLIT_335(x) (x) BOOST_PP_SEQ_SPLIT_334
# define BOOST_PP_SEQ_SPLIT_336(x) (x) BOOST_PP_SEQ_SPLIT_335
# define BOOST_PP_SEQ_SPLIT_337(x) (x) BOOST_PP_SEQ_SPLIT_336
# define BOOST_PP_SEQ_SPLIT_338(x) (x) BOOST_PP_SEQ_SPLIT_337
# define BOOST_PP_SEQ_SPLIT_339(x) (x) BOOST_PP_SEQ_SPLIT_338
# define BOOST_PP_SEQ_SPLIT_340(x) (x) BOOST_PP_SEQ_SPLIT_339
# define BOOST_PP_SEQ_SPLIT_341(x) (x) BOOST_PP_SEQ_SPLIT_340
# define BOOST_PP_SEQ_SPLIT_342(x) (x) BOOST_PP_SEQ_SPLIT_341
# define BOOST_PP_SEQ_SPLIT_343(x) (x) BOOST_PP_SEQ_SPLIT_342
# define BOOST_PP_SEQ_SPLIT_344(x) (x) BOOST_PP_SEQ_SPLIT_343
# define BOOST_PP_SEQ_SPLIT_345(x) (x) BOOST_PP_SEQ_SPLIT_344
# define BOOST_PP_SEQ_SPLIT_346(x) (x) BOOST_PP_SEQ_SPLIT_345
# define BOOST_PP_SEQ_SPLIT_347(x) (x) BOOST_PP_SEQ_SPLIT_346
# define BOOST_PP_SEQ_SPLIT_348(x) (x) BOOST_PP_SEQ_SPLIT_347
# define BOOST_PP_SEQ_SPLIT_349(x) (x) BOOST_PP_SEQ_SPLIT_348
# define BOOST_PP_SEQ_SPLIT_350(x) (x) BOOST_PP_SEQ_SPLIT_349
# define BOOST_PP_SEQ_SPLIT_351(x) (x) BOOST_PP_SEQ_SPLIT_350
# define BOOST_PP_SEQ_SPLIT_352(x) (x) BOOST_PP_SEQ_SPLIT_351
# define BOOST_PP_SEQ_SPLIT_353(x) (x) BOOST_PP_SEQ_SPLIT_352
# define BOOST_PP_SEQ_SPLIT_354(x) (x) BOOST_PP_SEQ_SPLIT_353
# define BOOST_PP_SEQ_SPLIT_355(x) (x) BOOST_PP_SEQ_SPLIT_354
# define BOOST_PP_SEQ_SPLIT_356(x) (x) BOOST_PP_SEQ_SPLIT_355
# define BOOST_PP_SEQ_SPLIT_357(x) (x) BOOST_PP_SEQ_SPLIT_356
# define BOOST_PP_SEQ_SPLIT_358(x) (x) BOOST_PP_SEQ_SPLIT_357
# define BOOST_PP_SEQ_SPLIT_359(x) (x) BOOST_PP_SEQ_SPLIT_358
# define BOOST_PP_SEQ_SPLIT_360(x) (x) BOOST_PP_SEQ_SPLIT_359
# define BOOST_PP_SEQ_SPLIT_361(x) (x) BOOST_PP_SEQ_SPLIT_360
# define BOOST_PP_SEQ_SPLIT_362(x) (x) BOOST_PP_SEQ_SPLIT_361
# define BOOST_PP_SEQ_SPLIT_363(x) (x) BOOST_PP_SEQ_SPLIT_362
# define BOOST_PP_SEQ_SPLIT_364(x) (x) BOOST_PP_SEQ_SPLIT_363
# define BOOST_PP_SEQ_SPLIT_365(x) (x) BOOST_PP_SEQ_SPLIT_364
# define BOOST_PP_SEQ_SPLIT_366(x) (x) BOOST_PP_SEQ_SPLIT_365
# define BOOST_PP_SEQ_SPLIT_367(x) (x) BOOST_PP_SEQ_SPLIT_366
# define BOOST_PP_SEQ_SPLIT_368(x) (x) BOOST_PP_SEQ_SPLIT_367
# define BOOST_PP_SEQ_SPLIT_369(x) (x) BOOST_PP_SEQ_SPLIT_368
# define BOOST_PP_SEQ_SPLIT_370(x) (x) BOOST_PP_SEQ_SPLIT_369
# define BOOST_PP_SEQ_SPLIT_371(x) (x) BOOST_PP_SEQ_SPLIT_370
# define BOOST_PP_SEQ_SPLIT_372(x) (x) BOOST_PP_SEQ_SPLIT_371
# define BOOST_PP_SEQ_SPLIT_373(x) (x) BOOST_PP_SEQ_SPLIT_372
# define BOOST_PP_SEQ_SPLIT_374(x) (x) BOOST_PP_SEQ_SPLIT_373
# define BOOST_PP_SEQ_SPLIT_375(x) (x) BOOST_PP_SEQ_SPLIT_374
# define BOOST_PP_SEQ_SPLIT_376(x) (x) BOOST_PP_SEQ_SPLIT_375
# define BOOST_PP_SEQ_SPLIT_377(x) (x) BOOST_PP_SEQ_SPLIT_376
# define BOOST_PP_SEQ_SPLIT_378(x) (x) BOOST_PP_SEQ_SPLIT_377
# define BOOST_PP_SEQ_SPLIT_379(x) (x) BOOST_PP_SEQ_SPLIT_378
# define BOOST_PP_SEQ_SPLIT_380(x) (x) BOOST_PP_SEQ_SPLIT_379
# define BOOST_PP_SEQ_SPLIT_381(x) (x) BOOST_PP_SEQ_SPLIT_380
# define BOOST_PP_SEQ_SPLIT_382(x) (x) BOOST_PP_SEQ_SPLIT_381
# define BOOST_PP_SEQ_SPLIT_383(x) (x) BOOST_PP_SEQ_SPLIT_382
# define BOOST_PP_SEQ_SPLIT_384(x) (x) BOOST_PP_SEQ_SPLIT_383
# define BOOST_PP_SEQ_SPLIT_385(x) (x) BOOST_PP_SEQ_SPLIT_384
# define BOOST_PP_SEQ_SPLIT_386(x) (x) BOOST_PP_SEQ_SPLIT_385
# define BOOST_PP_SEQ_SPLIT_387(x) (x) BOOST_PP_SEQ_SPLIT_386
# define BOOST_PP_SEQ_SPLIT_388(x) (x) BOOST_PP_SEQ_SPLIT_387
# define BOOST_PP_SEQ_SPLIT_389(x) (x) BOOST_PP_SEQ_SPLIT_388
# define BOOST_PP_SEQ_SPLIT_390(x) (x) BOOST_PP_SEQ_SPLIT_389
# define BOOST_PP_SEQ_SPLIT_391(x) (x) BOOST_PP_SEQ_SPLIT_390
# define BOOST_PP_SEQ_SPLIT_392(x) (x) BOOST_PP_SEQ_SPLIT_391
# define BOOST_PP_SEQ_SPLIT_393(x) (x) BOOST_PP_SEQ_SPLIT_392
# define BOOST_PP_SEQ_SPLIT_394(x) (x) BOOST_PP_SEQ_SPLIT_393
# define BOOST_PP_SEQ_SPLIT_395(x) (x) BOOST_PP_SEQ_SPLIT_394
# define BOOST_PP_SEQ_SPLIT_396(x) (x) BOOST_PP_SEQ_SPLIT_395
# define BOOST_PP_SEQ_SPLIT_397(x) (x) BOOST_PP_SEQ_SPLIT_396
# define BOOST_PP_SEQ_SPLIT_398(x) (x) BOOST_PP_SEQ_SPLIT_397
# define BOOST_PP_SEQ_SPLIT_399(x) (x) BOOST_PP_SEQ_SPLIT_398
# define BOOST_PP_SEQ_SPLIT_400(x) (x) BOOST_PP_SEQ_SPLIT_399
# define BOOST_PP_SEQ_SPLIT_401(x) (x) BOOST_PP_SEQ_SPLIT_400
# define BOOST_PP_SEQ_SPLIT_402(x) (x) BOOST_PP_SEQ_SPLIT_401
# define BOOST_PP_SEQ_SPLIT_403(x) (x) BOOST_PP_SEQ_SPLIT_402
# define BOOST_PP_SEQ_SPLIT_404(x) (x) BOOST_PP_SEQ_SPLIT_403
# define BOOST_PP_SEQ_SPLIT_405(x) (x) BOOST_PP_SEQ_SPLIT_404
# define BOOST_PP_SEQ_SPLIT_406(x) (x) BOOST_PP_SEQ_SPLIT_405
# define BOOST_PP_SEQ_SPLIT_407(x) (x) BOOST_PP_SEQ_SPLIT_406
# define BOOST_PP_SEQ_SPLIT_408(x) (x) BOOST_PP_SEQ_SPLIT_407
# define BOOST_PP_SEQ_SPLIT_409(x) (x) BOOST_PP_SEQ_SPLIT_408
# define BOOST_PP_SEQ_SPLIT_410(x) (x) BOOST_PP_SEQ_SPLIT_409
# define BOOST_PP_SEQ_SPLIT_411(x) (x) BOOST_PP_SEQ_SPLIT_410
# define BOOST_PP_SEQ_SPLIT_412(x) (x) BOOST_PP_SEQ_SPLIT_411
# define BOOST_PP_SEQ_SPLIT_413(x) (x) BOOST_PP_SEQ_SPLIT_412
# define BOOST_PP_SEQ_SPLIT_414(x) (x) BOOST_PP_SEQ_SPLIT_413
# define BOOST_PP_SEQ_SPLIT_415(x) (x) BOOST_PP_SEQ_SPLIT_414
# define BOOST_PP_SEQ_SPLIT_416(x) (x) BOOST_PP_SEQ_SPLIT_415
# define BOOST_PP_SEQ_SPLIT_417(x) (x) BOOST_PP_SEQ_SPLIT_416
# define BOOST_PP_SEQ_SPLIT_418(x) (x) BOOST_PP_SEQ_SPLIT_417
# define BOOST_PP_SEQ_SPLIT_419(x) (x) BOOST_PP_SEQ_SPLIT_418
# define BOOST_PP_SEQ_SPLIT_420(x) (x) BOOST_PP_SEQ_SPLIT_419
# define BOOST_PP_SEQ_SPLIT_421(x) (x) BOOST_PP_SEQ_SPLIT_420
# define BOOST_PP_SEQ_SPLIT_422(x) (x) BOOST_PP_SEQ_SPLIT_421
# define BOOST_PP_SEQ_SPLIT_423(x) (x) BOOST_PP_SEQ_SPLIT_422
# define BOOST_PP_SEQ_SPLIT_424(x) (x) BOOST_PP_SEQ_SPLIT_423
# define BOOST_PP_SEQ_SPLIT_425(x) (x) BOOST_PP_SEQ_SPLIT_424
# define BOOST_PP_SEQ_SPLIT_426(x) (x) BOOST_PP_SEQ_SPLIT_425
# define BOOST_PP_SEQ_SPLIT_427(x) (x) BOOST_PP_SEQ_SPLIT_426
# define BOOST_PP_SEQ_SPLIT_428(x) (x) BOOST_PP_SEQ_SPLIT_427
# define BOOST_PP_SEQ_SPLIT_429(x) (x) BOOST_PP_SEQ_SPLIT_428
# define BOOST_PP_SEQ_SPLIT_430(x) (x) BOOST_PP_SEQ_SPLIT_429
# define BOOST_PP_SEQ_SPLIT_431(x) (x) BOOST_PP_SEQ_SPLIT_430
# define BOOST_PP_SEQ_SPLIT_432(x) (x) BOOST_PP_SEQ_SPLIT_431
# define BOOST_PP_SEQ_SPLIT_433(x) (x) BOOST_PP_SEQ_SPLIT_432
# define BOOST_PP_SEQ_SPLIT_434(x) (x) BOOST_PP_SEQ_SPLIT_433
# define BOOST_PP_SEQ_SPLIT_435(x) (x) BOOST_PP_SEQ_SPLIT_434
# define BOOST_PP_SEQ_SPLIT_436(x) (x) BOOST_PP_SEQ_SPLIT_435
# define BOOST_PP_SEQ_SPLIT_437(x) (x) BOOST_PP_SEQ_SPLIT_436
# define BOOST_PP_SEQ_SPLIT_438(x) (x) BOOST_PP_SEQ_SPLIT_437
# define BOOST_PP_SEQ_SPLIT_439(x) (x) BOOST_PP_SEQ_SPLIT_438
# define BOOST_PP_SEQ_SPLIT_440(x) (x) BOOST_PP_SEQ_SPLIT_439
# define BOOST_PP_SEQ_SPLIT_441(x) (x) BOOST_PP_SEQ_SPLIT_440
# define BOOST_PP_SEQ_SPLIT_442(x) (x) BOOST_PP_SEQ_SPLIT_441
# define BOOST_PP_SEQ_SPLIT_443(x) (x) BOOST_PP_SEQ_SPLIT_442
# define BOOST_PP_SEQ_SPLIT_444(x) (x) BOOST_PP_SEQ_SPLIT_443
# define BOOST_PP_SEQ_SPLIT_445(x) (x) BOOST_PP_SEQ_SPLIT_444
# define BOOST_PP_SEQ_SPLIT_446(x) (x) BOOST_PP_SEQ_SPLIT_445
# define BOOST_PP_SEQ_SPLIT_447(x) (x) BOOST_PP_SEQ_SPLIT_446
# define BOOST_PP_SEQ_SPLIT_448(x) (x) BOOST_PP_SEQ_SPLIT_447
# define BOOST_PP_SEQ_SPLIT_449(x) (x) BOOST_PP_SEQ_SPLIT_448
# define BOOST_PP_SEQ_SPLIT_450(x) (x) BOOST_PP_SEQ_SPLIT_449
# define BOOST_PP_SEQ_SPLIT_451(x) (x) BOOST_PP_SEQ_SPLIT_450
# define BOOST_PP_SEQ_SPLIT_452(x) (x) BOOST_PP_SEQ_SPLIT_451
# define BOOST_PP_SEQ_SPLIT_453(x) (x) BOOST_PP_SEQ_SPLIT_452
# define BOOST_PP_SEQ_SPLIT_454(x) (x) BOOST_PP_SEQ_SPLIT_453
# define BOOST_PP_SEQ_SPLIT_455(x) (x) BOOST_PP_SEQ_SPLIT_454
# define BOOST_PP_SEQ_SPLIT_456(x) (x) BOOST_PP_SEQ_SPLIT_455
# define BOOST_PP_SEQ_SPLIT_457(x) (x) BOOST_PP_SEQ_SPLIT_456
# define BOOST_PP_SEQ_SPLIT_458(x) (x) BOOST_PP_SEQ_SPLIT_457
# define BOOST_PP_SEQ_SPLIT_459(x) (x) BOOST_PP_SEQ_SPLIT_458
# define BOOST_PP_SEQ_SPLIT_460(x) (x) BOOST_PP_SEQ_SPLIT_459
# define BOOST_PP_SEQ_SPLIT_461(x) (x) BOOST_PP_SEQ_SPLIT_460
# define BOOST_PP_SEQ_SPLIT_462(x) (x) BOOST_PP_SEQ_SPLIT_461
# define BOOST_PP_SEQ_SPLIT_463(x) (x) BOOST_PP_SEQ_SPLIT_462
# define BOOST_PP_SEQ_SPLIT_464(x) (x) BOOST_PP_SEQ_SPLIT_463
# define BOOST_PP_SEQ_SPLIT_465(x) (x) BOOST_PP_SEQ_SPLIT_464
# define BOOST_PP_SEQ_SPLIT_466(x) (x) BOOST_PP_SEQ_SPLIT_465
# define BOOST_PP_SEQ_SPLIT_467(x) (x) BOOST_PP_SEQ_SPLIT_466
# define BOOST_PP_SEQ_SPLIT_468(x) (x) BOOST_PP_SEQ_SPLIT_467
# define BOOST_PP_SEQ_SPLIT_469(x) (x) BOOST_PP_SEQ_SPLIT_468
# define BOOST_PP_SEQ_SPLIT_470(x) (x) BOOST_PP_SEQ_SPLIT_469
# define BOOST_PP_SEQ_SPLIT_471(x) (x) BOOST_PP_SEQ_SPLIT_470
# define BOOST_PP_SEQ_SPLIT_472(x) (x) BOOST_PP_SEQ_SPLIT_471
# define BOOST_PP_SEQ_SPLIT_473(x) (x) BOOST_PP_SEQ_SPLIT_472
# define BOOST_PP_SEQ_SPLIT_474(x) (x) BOOST_PP_SEQ_SPLIT_473
# define BOOST_PP_SEQ_SPLIT_475(x) (x) BOOST_PP_SEQ_SPLIT_474
# define BOOST_PP_SEQ_SPLIT_476(x) (x) BOOST_PP_SEQ_SPLIT_475
# define BOOST_PP_SEQ_SPLIT_477(x) (x) BOOST_PP_SEQ_SPLIT_476
# define BOOST_PP_SEQ_SPLIT_478(x) (x) BOOST_PP_SEQ_SPLIT_477
# define BOOST_PP_SEQ_SPLIT_479(x) (x) BOOST_PP_SEQ_SPLIT_478
# define BOOST_PP_SEQ_SPLIT_480(x) (x) BOOST_PP_SEQ_SPLIT_479
# define BOOST_PP_SEQ_SPLIT_481(x) (x) BOOST_PP_SEQ_SPLIT_480
# define BOOST_PP_SEQ_SPLIT_482(x) (x) BOOST_PP_SEQ_SPLIT_481
# define BOOST_PP_SEQ_SPLIT_483(x) (x) BOOST_PP_SEQ_SPLIT_482
# define BOOST_PP_SEQ_SPLIT_484(x) (x) BOOST_PP_SEQ_SPLIT_483
# define BOOST_PP_SEQ_SPLIT_485(x) (x) BOOST_PP_SEQ_SPLIT_484
# define BOOST_PP_SEQ_SPLIT_486(x) (x) BOOST_PP_SEQ_SPLIT_485
# define BOOST_PP_SEQ_SPLIT_487(x) (x) BOOST_PP_SEQ_SPLIT_486
# define BOOST_PP_SEQ_SPLIT_488(x) (x) BOOST_PP_SEQ_SPLIT_487
# define BOOST_PP_SEQ_SPLIT_489(x) (x) BOOST_PP_SEQ_SPLIT_488
# define BOOST_PP_SEQ_SPLIT_490(x) (x) BOOST_PP_SEQ_SPLIT_489
# define BOOST_PP_SEQ_SPLIT_491(x) (x) BOOST_PP_SEQ_SPLIT_490
# define BOOST_PP_SEQ_SPLIT_492(x) (x) BOOST_PP_SEQ_SPLIT_491
# define BOOST_PP_SEQ_SPLIT_493(x) (x) BOOST_PP_SEQ_SPLIT_492
# define BOOST_PP_SEQ_SPLIT_494(x) (x) BOOST_PP_SEQ_SPLIT_493
# define BOOST_PP_SEQ_SPLIT_495(x) (x) BOOST_PP_SEQ_SPLIT_494
# define BOOST_PP_SEQ_SPLIT_496(x) (x) BOOST_PP_SEQ_SPLIT_495
# define BOOST_PP_SEQ_SPLIT_497(x) (x) BOOST_PP_SEQ_SPLIT_496
# define BOOST_PP_SEQ_SPLIT_498(x) (x) BOOST_PP_SEQ_SPLIT_497
# define BOOST_PP_SEQ_SPLIT_499(x) (x) BOOST_PP_SEQ_SPLIT_498
# define BOOST_PP_SEQ_SPLIT_500(x) (x) BOOST_PP_SEQ_SPLIT_499
# define BOOST_PP_SEQ_SPLIT_501(x) (x) BOOST_PP_SEQ_SPLIT_500
# define BOOST_PP_SEQ_SPLIT_502(x) (x) BOOST_PP_SEQ_SPLIT_501
# define BOOST_PP_SEQ_SPLIT_503(x) (x) BOOST_PP_SEQ_SPLIT_502
# define BOOST_PP_SEQ_SPLIT_504(x) (x) BOOST_PP_SEQ_SPLIT_503
# define BOOST_PP_SEQ_SPLIT_505(x) (x) BOOST_PP_SEQ_SPLIT_504
# define BOOST_PP_SEQ_SPLIT_506(x) (x) BOOST_PP_SEQ_SPLIT_505
# define BOOST_PP_SEQ_SPLIT_507(x) (x) BOOST_PP_SEQ_SPLIT_506
# define BOOST_PP_SEQ_SPLIT_508(x) (x) BOOST_PP_SEQ_SPLIT_507
# define BOOST_PP_SEQ_SPLIT_509(x) (x) BOOST_PP_SEQ_SPLIT_508
# define BOOST_PP_SEQ_SPLIT_510(x) (x) BOOST_PP_SEQ_SPLIT_509
# define BOOST_PP_SEQ_SPLIT_511(x) (x) BOOST_PP_SEQ_SPLIT_510
# define BOOST_PP_SEQ_SPLIT_512(x) (x) BOOST_PP_SEQ_SPLIT_511
#
# endif
