//  (C) Copyright Gennadiy Rozental 2001.
//  Distributed under the Boost Software License, Version 1.0.
//  (See accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt)

//  See http://www.boost.org/libs/test for the library home page.
//
//  File        : $RCSfile$
//
//  Version     : $Revision$
//
//  Description : included variant of Execution Monitor to be used independently
// ***************************************************************************

#ifndef BOOST_INCLUDED_EXECUTION_MONITOR_HPP_051410GER
#define BOOST_INCLUDED_EXECUTION_MONITOR_HPP_051410GER

#include <boost/test/impl/execution_monitor.ipp>
#include <boost/test/impl/debug.ipp>

#endif // BOOST_INCLUDED_EXECUTION_MONITOR_HPP_051410GER
