//
// Copyright (c) 2019-2025 Ruben Perez Hidalgo (rubenperez038 at gmail dot com)
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
//

#ifndef BOOST_MYSQL_DETAIL_SSL_FWD_HPP
#define BOOST_MYSQL_DETAIL_SSL_FWD_HPP

// SSL headers are heavyweight
namespace boost {
namespace asio {
namespace ssl {

class context;

}  // namespace ssl
}  // namespace asio
}  // namespace boost

#endif
