// Copyright (C) 2020 T. Zachary Laine
//
// Distributed under the Boost Software License, Version 1.0. (See
// accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)
#ifndef BOOST_PARSER_DETAIL_TEXT_DETAIL_SENTINEL_TAG_HPP
#define BOOST_PARSER_DETAIL_TEXT_DETAIL_SENTINEL_TAG_HPP

#include <boost/parser/detail/text/config.hpp>


namespace boost::parser::detail { namespace text { namespace detail {
    struct sentinel_tag
    {};
    struct non_sentinel_tag
    {};
}}}

#endif
