// Copyright (C) 2016-2018 T. Zachary Laine
//
// Distributed under the Boost Software License, Version 1.0. (See
// accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)
#ifndef BOOST_YAP_YAP_HPP_INCLUDED
#define BOOST_YAP_YAP_HPP_INCLUDED


#include <boost/yap/algorithm.hpp>
#include <boost/yap/expression.hpp>

#endif
