/*=============================================================================
    Phoenix V1.2.1
    Copyright (c) 2001-2002 Joel de Guzman

  Distributed under the Boost Software License, Version 1.0. (See accompanying
  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
==============================================================================*/
#if !defined(BOOST_SPIRIT_PHOENIX_HPP)
#define BOOST_SPIRIT_PHOENIX_HPP

#include <boost/spirit/home/classic/phoenix/tuples.hpp>
#include <boost/spirit/home/classic/phoenix/tuple_helpers.hpp>
#include <boost/spirit/home/classic/phoenix/actor.hpp>
#include <boost/spirit/home/classic/phoenix/primitives.hpp>
#include <boost/spirit/home/classic/phoenix/composite.hpp>
#include <boost/spirit/home/classic/phoenix/functions.hpp>
#include <boost/spirit/home/classic/phoenix/operators.hpp>
#include <boost/spirit/home/classic/phoenix/special_ops.hpp>
#include <boost/spirit/home/classic/phoenix/statements.hpp>
#include <boost/spirit/home/classic/phoenix/binders.hpp>
#include <boost/spirit/home/classic/phoenix/closures.hpp>
#include <boost/spirit/home/classic/phoenix/casts.hpp>
#include <boost/spirit/home/classic/phoenix/new.hpp>

#endif // !defined(BOOST_SPIRIT_PHOENIX_HPP)
