// Copyright Vladimir Prus 2002.
// Distributed under the Boost Software License, Version 1.0. (See
// accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

//  See www.boost.org/libs/program_options for documentation.

#ifndef BOOST_PROGRAM_OPTIONS_VP_2003_05_19
#define BOOST_PROGRAM_OPTIONS_VP_2003_05_19

#if defined(_MSC_VER)
#pragma once
#endif

#include <boost/program_options/options_description.hpp>
#include <boost/program_options/positional_options.hpp>
#include <boost/program_options/parsers.hpp>
#include <boost/program_options/variables_map.hpp>
#include <boost/program_options/cmdline.hpp>
#include <boost/program_options/errors.hpp>
#include <boost/program_options/option.hpp>
#include <boost/program_options/value_semantic.hpp>
#include <boost/program_options/version.hpp>

#endif // BOOST_PROGRAM_OPTIONS_VP_2003_05_19
